import numpy as np
import pygame

pygame.init()

TMP = "/tmp"

BLACK = np.array((0, 0, 0))
DARK_GRAY = np.array((50, 50, 50))
GRAY = np.array((120, 120, 120))
LIGHT_GRAY = np.array((170, 170, 170))
WHITE = np.array((255, 255, 255))
BLUE = np.array((130, 130, 255))

FONT_MED = pygame.font.SysFont("ubuntu", 18)
